from mle_scheduler.cluster.slurm.helpers_launch_slurm import slurm_generate_startup_file

job_arguments = {
    "num_logical_cores": 5,
    "partition": "standard",
    "job_name": "test_job",
    "num_gpus": 1,
    "gpu_type": "RTX2080",
    "env_name": "test_env",
    "use_conda_venv": True,
    "script": "python run.py",
    "memory_per_cpu": 2000,
    "time_per_job": "10:05:02",
    "modules_to_load": "nvidia/cuda/10.0",
}

job_script = """#!/bin/bash
#SBATCH --job-name=test_job
#SBATCH --output=log.txt
#SBATCH --error=err.err
#SBATCH --partition=standard
#SBATCH --cpus-per-task=5
#SBATCH --gres=gpu:RTX2080:1
#SBATCH --mem-per-cpu=2000
#SBATCH --time=10:05:02

echo "------------------------------------------------------------------------"
source ~/miniconda3/etc/profile.d/conda.sh
echo "------------------------------------------------------------------------"
. ~/.bashrc && conda activate test_env
echo "Successfully activated virtual environment - Ready to start job"

module load nvidia/cuda/10.0
echo "------------------------------------------------------------------------"
echo "Job started on" `date`
echo "------------------------------------------------------------------------"
python run.py
echo "------------------------------------------------------------------------"
echo "Job ended on" `date`
echo "------------------------------------------------------------------------"
conda deactivate
"""


def test_job_slurm():
    startup_script = slurm_generate_startup_file(job_arguments).format(**job_arguments)
    assert job_script == startup_script
    return



import os
from typing import Union


def ssh_get_submission_cmd(
    filename: str,
    cmd_line_arguments: Union[str, None],
    job_arguments: dict,
    ssh_settings: dict,
):
    """Create shell script string to execute on remote SSH server."""
    if cmd_line_arguments is None:
        cmd_line_arguments = ""
    # Write the desired python/bash execution to slurm job submission file
    f_name, f_extension = os.path.splitext(filename)
    if f_extension == ".py":
        cmd = f"python {filename} {cmd_line_arguments}"
    elif f_extension == ".sh":
        cmd = f"bash {filename} {cmd_line_arguments}"
    else:
        raise ValueError(
            f"Script with {f_extension} cannot be handled"
            " by mle-toolbox. Only base .py, .sh experiments"
            " are so far implemented. Please open an issue."
        )

    # Add conda environment activation
    if "use_conda_venv" in job_arguments:
        if job_arguments["use_conda_venv"]:
            script_cmd = "echo $$; /bin/bash -c 'source $(conda info --base)/etc/profile.d/conda.sh && conda activate {} && cd {} && {}'".format(
                job_arguments["env_name"], ssh_settings["remote_dir"], cmd
            )
        else:
            script_cmd = "echo $$; /bin/bash -c 'cd {} && {}'".format(
                ssh_settings["remote_dir"], cmd
            )
    elif "use_venv_venv" in job_arguments:
        if job_arguments["use_venv_venv"]:
            script_cmd = "echo $$; /bin/bash -c 'source {}/{}/bin/activate && cd {} && {}'".format(
                os.environ["WORKON_HOME"],
                job_arguments["env_name"],
                ssh_settings["remote_dir"],
                cmd,
            )
        else:
            script_cmd = "echo $$; /bin/bash -c 'cd {} && {}'".format(
                ssh_settings["remote_dir"], cmd
            )
    else:
        script_cmd = "echo $$; /bin/bash -c 'cd {} && {}'".format(
            ssh_settings["remote_dir"], cmd
        )
    return script_cmd



from mle_scheduler.cloud.gcp.helpers_launch_gcp import gcp_get_submission_cmd

vm_name = "temp-vm"
job_arguments = {
    "use_tpus": 0,
    "num_gpus": 0,
    "gpu_type": "nvidia-tesla-v100",
    "num_logical_cores": 1,
    "job_name": "test",
}
startup_fname = "temp-vm"

correct_gcp_launch_cmd = [
    "gcloud",
    "compute",
    "instances",
    "create",
    "temp-vm",
    "--preemptible",
    "--zone=us-west1-a",
    "--custom-cpu=2",
    "--custom-memory=6144MB",
    "--custom-vm-type=n1",
    "--image=c1-deeplearning-tf-2-4-cu110-v20210414-debian-10",
    "--image-project=ml-images",
    "--metadata-from-file=startup-script=temp-vm.sh",
    "--scopes=cloud-platform,storage-full",
    "--boot-disk-size=128GB",
    "--boot-disk-type=pd-standard",
    "--no-user-output-enabled",
    "--verbosity",
    "error",
]
correct_job_gcp_args = {
    "ZONE": "us-west1-a",
    "ACCELERATOR_TYPE": None,
    "ACCELERATOR_COUNT": 0,
    "MACHINE_TYPE": "n2-highcpu-8",
    "IMAGE_NAME": "c1-deeplearning-tf-2-4-cu110-v20210414-debian-10",
    "IMAGE_PROJECT": "ml-images",
}


def test_submission_cmd_gcp():
    gcp_launch_cmd, job_gcp_args = gcp_get_submission_cmd(
        vm_name, job_arguments, startup_fname
    )
    return



import os
from typing import Union
from .ssh_manager import SSH_Manager


def send_dir_ssh(
    ssh_settings: dict,
    local_dir: Union[str, None] = None,
):
    """Helper for sending code directory to SSH server."""
    if local_dir is None:
        local_dir = os.getcwd()
    # Create manager for file sync and subprocess exec
    ssh_manager = SSH_Manager(
        user_name=ssh_settings["user_name"],
        pkey_path=ssh_settings["pkey_path"],
        main_server=ssh_settings["main_server"],
        jump_server=ssh_settings["jump_server"],
        ssh_port=ssh_settings["ssh_port"],
    )
    ssh_manager.sync_dir(local_dir, ssh_settings["remote_dir"])
    return


def copy_dir_ssh(
    ssh_settings: dict,
    remote_dir: str,
    local_dir: Union[str, None] = None,
):
    """Helper for copying results directory back to local machine."""
    # Create manager for file sync and subprocess exec
    ssh_manager = SSH_Manager(
        user_name=ssh_settings["user_name"],
        pkey_path=ssh_settings["pkey_path"],
        main_server=ssh_settings["main_server"],
        jump_server=ssh_settings["jump_server"],
        ssh_port=ssh_settings["ssh_port"],
    )
    if local_dir is None:
        ssh_manager.get_file(remote_dir, os.getcwd())
    else:
        ssh_manager.get_file(remote_dir, local_dir)
    return


def delete_dir_ssh(ssh_settings: dict):
    """Delete log.txt file."""
    # Create manager for file sync and subprocess exec
    ssh_manager = SSH_Manager(
        user_name=ssh_settings["user_name"],
        pkey_path=ssh_settings["pkey_path"],
        main_server=ssh_settings["main_server"],
        jump_server=ssh_settings["jump_server"],
        ssh_port=ssh_settings["ssh_port"],
    )
    ssh_manager.delete_dir(ssh_settings["remote_dir"])



from .job_manage_gcp import submit_gcp, monitor_gcp, clean_up_gcp
from .file_manage_gcp import send_dir_gcp, copy_dir_gcp, delete_dir_gcp

__all__ = [
    "submit_gcp",
    "monitor_gcp",
    "clean_up_gcp",
    "send_dir_gcp",
    "copy_dir_gcp",
    "delete_dir_gcp",
]



from .startup_script_slurm import (
    slurm_base_job_config,
    slurm_job_exec,
    slurm_conda_activate,
    slurm_venv_activate,
)


def slurm_generate_startup_file(job_arguments: dict) -> str:
    """Generate the bash script template to submit with SBATCH."""
    # Set the job template depending on the desired number of GPUs
    base_template = (slurm_base_job_config + ".")[:-1]

    # Add desired number of requested gpus
    if "num_gpus" in job_arguments:
        if job_arguments["num_gpus"] > 0:
            if job_arguments["gpu_type"] is not None:
                base_template += "#SBATCH --gres=gpu:{gpu_type}:{num_gpus}\n"
            else:
                base_template += "#SBATCH --gres=gpu:{num_gpus}\n"

    # Set the max required memory per job
    if "memory_per_cpu" in job_arguments:
        base_template += "#SBATCH --mem-per-cpu={memory_per_cpu}\n"

    # Set the max required time per job (afterwards terminate)
    if "time_per_job" in job_arguments:
        base_template += "#SBATCH --time={time_per_job}\n"

    # Set the max required memory per job in MB - standardization slurm
    if "memory_per_job" in job_arguments:
        base_template += "#SBATCH --mem={memory_per_job}\n"

    # Add the 'tail' - script execution to the string
    template_out = base_template
    if "use_conda_venv" in job_arguments:
        if job_arguments["use_conda_venv"]:
            template_out += slurm_conda_activate
    elif "use_venv_venv" in job_arguments:
        if job_arguments["use_venv_venv"]:
            template_out += slurm_venv_activate

    # Add optional modules to load (e.g. CUDA, etc.)
    if "modules_to_load" in job_arguments:
        if type(job_arguments["modules_to_load"]) == str:
            template_out += f"\nmodule load {job_arguments['modules_to_load']}"
        elif type(job_arguments["modules_to_load"]) == list:
            for mod in job_arguments["modules_to_load"]:
                template_out += f"\nmodule load {job_arguments['mod']}"
    template_out += slurm_job_exec
    return template_out



from .ssh_manager import SSH_Manager
from .job_manage_ssh import submit_ssh, monitor_ssh
from .file_manage_ssh import send_dir_ssh, copy_dir_ssh, delete_dir_ssh


__all__ = [
    "SSH_Manager",
    "submit_ssh",
    "monitor_ssh",
    "send_dir_ssh",
    "copy_dir_ssh",
    "delete_dir_ssh",
]



import os
import time
import subprocess as sp
from typing import Union
from ...local import submit_subprocess, random_id
from .helpers_launch_slurm import slurm_generate_startup_file


def submit_slurm(
    filename: str,
    cmd_line_arguments: str,
    job_arguments: dict,
    user_name: str,
    debug_mode: bool,
    clean_up: bool = True,
) -> str:
    """Create a qsub job & submit it based on provided file to execute."""
    # Create base string of job id
    base = "submit_{0}".format(random_id())

    # Write the desired python/bash execution to slurm job submission file
    f_name, f_extension = os.path.splitext(filename)
    if f_extension == ".py":
        script = f"python {filename} {cmd_line_arguments}"
    elif f_extension == ".sh":
        script = f"bash {filename} {cmd_line_arguments}"
    else:
        raise ValueError(
            f"Script with {f_extension} cannot be handled"
            " by mle-toolbox. Only base .py, .sh experiments"
            " are so far implemented. Please open an issue."
        )
    job_arguments["script"] = script
    slurm_job_template = slurm_generate_startup_file(job_arguments)

    # Create combined string if multiple jobs provided as list
    if type(job_arguments["partition"]) == list:
        job_arguments["partition"] = ",".join(job_arguments["partition"])

    # Add path for virtualenv activation
    if "use_venv_venv" in job_arguments:
        if job_arguments["use_venv_venv"]:
            job_arguments["work_on_dir"] = os.environ["WORKON_HOME"]

    # Reformatting of time for Slurm SBASH - d-hh:mm but in is dd:hh:mm
    if "time_per_job" in job_arguments:
        days, hours, minutes = job_arguments["time_per_job"].split(":")
        slurm_time = days[1] + "-" + hours + ":" + minutes
        job_arguments["time_per_job"] = slurm_time

    # Add job name if not given in arguments
    if "job_name" not in job_arguments:
        job_arguments["job_name"] = "job"

    # Write slurm scheduling script to bash file
    open(base + ".sh", "w").write(slurm_job_template.format(**job_arguments))

    # Submit the job via subprocess call
    command = "sbatch < " + base + ".sh"
    while True:
        proc = submit_subprocess(command, debug_mode)

        # Wait until system has processed submission
        while True:
            poll = proc.poll()
            if poll is None:
                continue
            else:
                break

        # Get output & error messages (if there is an error)
        out, err = proc.communicate()
        if proc.returncode != 0:
            print(out, err)
            job_id = -1
        else:
            # print(out.decode("utf-8").split())
            job_id = int(out.decode("utf-8").split()[-1])
            break
    # Wait until the job is listed under the qstat scheduled jobs
    while True:
        job_running = monitor_slurm(job_id, user_name)
        if job_running:
            break
    # Finally delete all the unneccessary submission file
    if clean_up:
        os.remove(base + ".sh")

    return job_id


def monitor_slurm(job_id: Union[list, int], user_name: str) -> bool:
    """Monitor the status of a job based on its id."""
    while True:
        try:
            out = sp.check_output(["squeue", "-u", user_name])
            break
        except sp.CalledProcessError as e:
            stderr = e.stderr
            return_code = e.returncode
            print(stderr, return_code)
            time.sleep(0.5)

    job_info = out.split(b"\n")[1:]
    running_job_ids = [
        int(job_info[i].decode("utf-8").split()[0])
        for i in range(len(job_info) - 1)
    ]
    if type(job_id) == int:
        job_id = [job_id]
    S1 = set(job_id)
    S2 = set(running_job_ids)
    job_status = len(S1.intersection(S2)) > 0
    return job_status



from mle_scheduler.ssh.helpers_launch_ssh import ssh_get_submission_cmd

filename = "train.py"
cmd_line_arguments = "-exp_dir logs_ssh_single -config base_config_1.yaml"
job_arguments = {"env_name": "mle-toolbox", "use_conda_venv": True}
ssh_settings = {
    "remote_dir": "mle-code-dir",
}


correct_cmd = "echo $$; /bin/bash -c 'source $(conda info --base)/etc/profile.d/conda.sh && conda activate mle-toolbox && cd mle-code-dir && python train.py -exp_dir logs_ssh_single -config base_config_1.yaml'"


def test_submission_cmd_ssh():
    script_cmd = ssh_get_submission_cmd(
        filename, cmd_line_arguments, job_arguments, ssh_settings
    )
    assert script_cmd == correct_cmd



import subprocess as sp
import time
import datetime
import os
import random
import re
from .helpers_launch_gcp import (
    gcp_generate_startup_file,
    gcp_get_submission_cmd,
    gcp_delete_vm_instance,
)


def submit_gcp(
    filename: str,
    cmd_line_arguments: str,
    experiment_dir: str,
    job_arguments: dict,
    debug_mode: bool,
    cloud_settings: dict,
):
    """Create a GCP VM job & submit it based on provided file to execute."""
    if "job_name" not in job_arguments:
        job_arguments["job_name"] = "job"

    if "use_tpus" not in job_arguments:
        job_arguments["use_tpus"] = 0

    if "num_gpus" not in job_arguments:
        job_arguments["num_gpus"] = 0

    # 0. Create VM Name - Timestamp + Random 4 digit id
    timestamp = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")
    random_str = str(random.randrange(1000, 9999))
    vm_name = "-".join(["mle", timestamp, random_str])
    vm_name = re.sub(r"[^a-z0-9-]", "-", vm_name)

    # 1. Generate GCP startup file with provided arguments
    startup_fname = vm_name + "-startup.sh"
    if "extra_install_fname" in job_arguments:
        extra_install_fname = job_arguments["extra_install_fname"]
    else:
        extra_install_fname = None

    gcp_generate_startup_file(
        cloud_settings["remote_dir"],
        cloud_settings["bucket_name"],
        filename,
        experiment_dir,
        startup_fname,
        cmd_line_arguments,
        extra_install_fname,
        job_arguments["use_tpus"],
        job_arguments["num_gpus"] > 0,
    )

    # 2. Generate GCP submission command (`gcloud compute instance create ...`)
    gcp_launch_cmd, job_gcp_args = gcp_get_submission_cmd(
        vm_name, job_arguments, startup_fname
    )

    if debug_mode:
        gcp_launch_cmd += " 2>>err.err 1>>log.txt"

    # 3. Launch GCP VM Instance - Everything handled by startup file
    sp.run(gcp_launch_cmd)

    # 4. Wait until job is listed as running (ca. 2 minute!)
    while True:
        try:
            job_running = monitor_gcp(vm_name, job_arguments)
            if job_running == 1:
                # Delete statup bash file
                try:
                    os.remove(startup_fname)
                except Exception:
                    pass
                return vm_name
            time.sleep(10)
        except sp.CalledProcessError as e:
            stderr = e.stderr
            return_code = e.returncode
            print(stderr, return_code)
            time.sleep(2)

    # Return -1 if job was not listed as starting/running within 100 seconds!
    return -1


def monitor_gcp(vm_name: str, job_arguments: dict):
    """Monitor status of job based on vm_name. Requires stable connection."""
    # Check VM status from command line
    while True:
        try:
            check_cmd = (
                [
                    "gcloud",
                    "alpha",
                    "compute",
                    "tpus",
                    "--zone",
                    "europe-west4-a",
                    "--verbosity",
                    "critical",
                ]
                if job_arguments["use_tpus"]
                else [
                    "gcloud",
                    "compute",
                    "instances",
                    "list",
                    "--verbosity",
                    "critical",
                ]
            )
            out = sp.check_output(check_cmd)
            break
        except sp.CalledProcessError as e:
            stderr = e.stderr
            return_code = e.returncode
            print(stderr, return_code)
            time.sleep(1)

    # Clean up and check if vm_name is in list of all jobs
    job_info = out.split(b"\n")[1:-1]
    running_job_names = []
    for i in range(len(job_info)):
        decoded_job_info = job_info[i].decode("utf-8").split()
        if decoded_job_info[-1] in ["STAGING", "RUNNING"]:
            running_job_names.append(decoded_job_info[0])
    job_status = vm_name in running_job_names
    return job_status


def clean_up_gcp(
    vm_name: str,
    job_arguments: dict,
    experiment_dir: str,
    cloud_settings: dict,
):
    """Delete VM instance and code GCS directory."""
    # Delete GCP Job after it terminated (avoid storage billing)
    gcp_delete_vm_instance(vm_name, job_arguments["use_tpus"])



import os
import subprocess as sp
from dotmap import DotMap
from typing import Union, Tuple
from .startup_script_gcp import (
    tmux_setup,
    clone_gcp_bucket_dir,
    install_venv,
    install_additional_setup,
    jax_gpu_build,
    jax_tpu_build,
    exec_python,
    exec_bash,
    sync_results_from_dir,
)


cores_to_machine_type = {
    1: "n1-highcpu-2",
    2: "n1-highcpu-4",
    4: "n1-highcpu-8",
    8: "n1-highcpu-16",
    16: "n1-highcpu-32",
    32: "n1-highcpu-64",
    48: "n1-highcpu-96",
}

valid_gpu_types = [
    "nvidia-tesla-p100",
    "nvidia-tesla-v100",
    "nvidia-tesla-t4",
    "nvidia-tesla-p4",
    "nvidia-tesla-k80",
]


base_gcp_args = dict(
    {
        "ZONE": "us-west1-a",
        "ACCELERATOR_TYPE": None,
        "ACCELERATOR_COUNT": 0,
        "MACHINE_TYPE": "n2-highcpu-8",
        "IMAGE_NAME": "c1-deeplearning-tf-2-4-cu110-v20210414-debian-10",
        "IMAGE_PROJECT": "ml-images",
    }
)


tpu_gcp_args = dict(
    {
        "ZONE": "europe-west4-a",
        "ACCELERATOR_TYPE": "v3-8",
        "RUNTIME_VERSION": "v2-alpha",
    }
)


def gcp_get_submission_cmd(
    vm_name: str, job_args: DotMap, startup_fname: str
) -> Tuple[list, dict]:
    """Construct gcloud VM instance creation cmd to execute via cmd line."""
    if job_args["use_tpus"]:
        job_gcp_args = tpu_gcp_args
    else:
        job_gcp_args = base_gcp_args
        # job_gcp_args.MACHINE_TYPE = cores_to_machine_type[
        #                                 job_args.num_logical_cores]
        if job_args["num_gpus"] > 0:
            assert job_args["gpu_type"] in valid_gpu_types
            job_gcp_args["ACCELERATOR_TYPE"] = job_args["gpu_type"]
            job_gcp_args["ACCELERATOR_COUNT"] = job_args["num_gpus"]

    if job_args["use_tpus"]:
        # TPU VM Alpha gcloud create CMD
        gcp_launch_cmd = [
            "gcloud",
            "alpha",
            "compute",
            "tpus",
            "tpu-vm",
            "create",
            f"{vm_name}",
            "--preemptible",
            f"--zone={job_gcp_args['ZONE']}",
            f"--accelerator-type={job_gcp_args['ACCELERATOR_TYPE']}",
            f"--version={job_gcp_args['RUNTIME_VERSION']}",
            f"--metadata-from-file=startup-script={startup_fname}",
            "--no-user-output-enabled",
            "--verbosity",
            "error",
        ]
    else:
        # CPU VM gcloud create CMD w/o any GPU attached
        gcp_launch_cmd = [
            "gcloud",
            "compute",
            "instances",
            "create",
            f"{vm_name}",
            "--preemptible",
            f"--zone={job_gcp_args['ZONE']}",
            f"--custom-cpu={2*job_args['num_logical_cores']}",
            f"--custom-memory={4096 + 2048*job_args['num_logical_cores']}MB",
            "--custom-vm-type=n1",
            f"--image={job_gcp_args['IMAGE_NAME']}",
            f"--image-project={job_gcp_args['IMAGE_PROJECT']}",
            f"--metadata-from-file=startup-script={startup_fname}",
            "--scopes=cloud-platform,storage-full",
            "--boot-disk-size=128GB",
            "--boot-disk-type=pd-standard",
            "--no-user-output-enabled",
            "--verbosity",
            "error",
        ]

        # Attach GPUs to Job if desired - make sure to install nvidia driver
        if (
            job_gcp_args["ACCELERATOR_COUNT"] > 0
            and job_gcp_args["ACCELERATOR_TYPE"] is not None
        ):
            gcp_launch_cmd += [
                "--metadata=install-nvidia-driver=True"
                "--maintenance-policy=TERMINATE",
                f"--accelerator=type={job_gcp_args['ACCELERATOR_TYPE']},"
                f"count={job_gcp_args['ACCELERATOR_COUNT']}",
            ]

    return gcp_launch_cmd, job_gcp_args


def gcp_generate_startup_file(
    remote_code_dir: str,
    gcp_bucket_name: str,
    job_filename: str,
    experiment_dir: str,
    startup_fname: str,
    cmd_line_arguments: str,
    extra_install_fname: Union[None, str],
    use_tpus: bool = False,
    use_cuda: bool = False,
) -> None:
    """Generate bash script template to launch at VM startup."""
    # Build the start job execution script
    # 1. Connecting to tmux via: gcloud compute ssh $VM -- /sudo_tmux_a.sh
    # 2a. Launch venv & install dependencies from requirements.txt
    # 2b. [OPTIONAL] Setup JAX TPU/GPU build
    # 3. Separate tmux split for rsync of results to GCS bucket
    startup_script_content = (
        "#!/bin/bash"
        + tmux_setup
        + clone_gcp_bucket_dir.format(
            remote_dir=remote_code_dir, gcp_bucket_name=gcp_bucket_name
        )
        + install_venv.format(remote_dir=remote_code_dir)
    )

    if extra_install_fname is not None:
        # Bash execute .sh file for additional requirements (if specified)
        startup_script_content += install_additional_setup.format(extra_install_fname)

    if use_tpus:
        # Install TPU version JAX
        startup_script_content += jax_tpu_build
    elif use_cuda:
        # Install GPU version JAX
        startup_script_content += jax_gpu_build

    # Write the desired python/bash execution to slurm job submission file
    f_name, f_extension = os.path.splitext(job_filename)
    if f_extension == ".py":
        startup_script_content += exec_python.format(
            remote_dir=remote_code_dir,
            filename=job_filename,
            cmd_line_arguments=cmd_line_arguments,
        )
    elif f_extension == ".sh":
        startup_script_content += exec_bash.format(
            remote_dir=remote_code_dir,
            filename=job_filename,
            cmd_line_arguments=cmd_line_arguments,
        )
    else:
        raise ValueError(
            f"Script with {f_extension} cannot be handled"
            " by mle-toolbox. Only base .py, .sh experiments"
            " are so far implemented. Please open an issue."
        )

    startup_script_content += sync_results_from_dir.format(
        remote_code_dir=remote_code_dir,
        gcp_bucket_name=gcp_bucket_name,
        experiment_dir=experiment_dir,
    )

    # Write startup script to physical file
    with open(startup_fname, "w", encoding="utf8") as f:
        f.write(startup_script_content)


def gcp_delete_vm_instance(vm_name: str, use_tpus: bool = False) -> None:
    """Quitely delete job by its name + zone. TODO: Add robustness check."""
    vm_zone = "europe-west4-a" if use_tpus else "us-west1-a"
    if not use_tpus:
        gcp_delete_cmd = [
            "gcloud",
            "compute",
            "instances",
            "delete",
            f"{vm_name}",
            "--zone",
            f"{vm_zone}",
            "--quiet",
            "--no-user-output-enabled",
            "--verbosity",
            "error",
        ]
    else:
        gcp_delete_cmd = [
            "gcloud",
            "alpha",
            "compute",
            "tpus",
            "tpu-vm",
            "delete",
            f"{vm_name}",
            "--zone",
            f"{vm_zone}",
            "--quiet",
            "--no-user-output-enabled",
            "--verbosity",
            "error",
        ]
    sp.run(gcp_delete_cmd)



# Useful string lego building blocks for GCP startup file formatting
#   1. Copy code directory from GCS bucket
#   2. Setting up tmux session (a) htop (b) startup exec (c) GCS rsync results
#   3. Installation of venv, requirements and jaxlib accelerator dependencies
#   4. Python Base Job File Execution
#   5. Sync Results with GCS bucket

# Inspired by the flax GCP example: https://github.com/google/flax/tree/main/examples/cloud

clone_gcp_bucket_dir = """
mkdir {remote_dir}
sudo chmod 777 {remote_dir}
gsutil cp -r gs://{gcp_bucket_name}/{remote_dir} .
"""

tmux_setup = """
# Login directly with:
# gcloud compute ssh $VM -- sudo_tmux_a.sh
echo -e '#!/bin/bash\nsudo tmux_a.sh' > sudo_tmux_a.sh
chmod a+x sudo_tmux_a.sh
echo -e '#!/bin/bash\ntmux a' > tmux_a.sh
chmod a+x /tmux_a.sh
"""

install_venv = """# Setup virtual env + install base required packages
tmux new-session -s gcp_exp -d htop ENTER
tmux split-window
tmux send "
    set -x
    [ -d gcp_exp ] || (
    cd {remote_dir}
    python3 -m pip install virtualenv
    python3 -m virtualenv env
    . env/bin/activate
    pip install -U pip
    pip install -r requirements.txt
    ) 2>&1 | tee -a log_startup.txt
"
"""

install_additional_setup = """# Setup virtual env + install base required packages
tmux send "
    cd {remote_dir}
    . env/bin/activate
    bash {extra_install_fname}
    "
"""

jax_tpu_build = """# Install jaxlib for TPU
tmux send "
    cd {remote_dir}
    . env/bin/activate
    PYTHON_VERSION=cp36  # Supported python versions: cp36, cp37, cp38
    pip install --upgrade --user https://storage.googleapis.com/jax-releases/tpu/jaxlib-0.1.55+tpu-$PYTHON_VERSION-none-manylinux2010_x86_64.whl
    pip install --upgrade --user jax
    "
"""

jax_gpu_build = """# Install jaxlib for GPU
tmux send "
    cd {remote_dir}
    . env/bin/activate
    pip install --upgrade jax jaxlib==0.1.64+cuda110 -f https://storage.googleapis.com/jax-releases/jax_releases.html
    "
"""

exec_python = """
tmux send "
(
  cd {remote_dir} &&
  . env/bin/activate &&
  python3 {filename} {cmd_line_arguments}
) 2>&1 | tee -a {remote_dir}/log.txt

echo WILL SHUT DOWN IN 5 MIN ...
sleep 100 && sudo shutdown now
"
"""

exec_bash = """
tmux send "
(
  cd {remote_dir} &&
  . env/bin/activate &&
  bash {filename} {cmd_line_arguments}
) 2>&1 | tee -a {remote_dir}/log.txt

echo WILL SHUT DOWN IN 5 MIN ...
sleep 100 && sudo shutdown now
"
"""

sync_results_from_dir = """
# Wait for experiment startup before continuous rsync
sleep 120
tmux split-window -h
tmux send "
while true; do
    gsutil rsync -x 'env' -r {remote_code_dir}/{experiment_dir} gs://{gcp_bucket_name}/{remote_code_dir}/{experiment_dir}
    sleep 10
done
" ENTER
"""
